/*
 * $Header: /cvs/lusers/utmpfns.c,v 1.1 2001/04/30 11:13:53 richard Exp $
 *
 *    utmp.c - utmp reading code for systems without setutent etc
 *    Copyright (C) 2001 Richard Kettlewell
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#if ! HAVE_SETUTENT

#include <time.h>
#include <stdio.h>
#include <utmp.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <limits.h>

#include "utmpfns.h"

/* re-entrant interface, made up by RJK */

int setutent_r(FILE **fpp) {
  if(*fpp)
    if(fseek(*fpp, 0, SEEK_SET) < 0)
      return errno;
  return 0;
}

int endutent_r(FILE **fpp) {
  if(*fpp)
    if(fclose(*fpp) < 0)
      return errno;
  return 0;
}

int getutent_r(const char *path,
	       FILE **fpp,
	       struct utmp *utmp_entry) {
  size_t n;
  int c;
  
  if(!*fpp
     && !(*fpp = fopen(path, "rb")))
    return errno;
  do {
    for(n = 0; n < sizeof (struct utmp); ++n) {
      if((c = fgetc(*fpp)) < 0)
	return ferror(*fpp) ? errno : -1;
      ((char *)utmp_entry)[n] = c;
    }
  } while(utmp_entry->ut_name[0] == 0);
  return 0;
}

/* unsafe interface as found in SysV and GNU */

static FILE *utmp_fp;
static const char utmp_path[PATH_MAX] = _PATH_UTMP;

void setutent(void) {
  setutent_r(&utmp_fp);
}

void endutent(void) {
  endutent_r(&utmp_fp);
}

struct utmp *getutent(void) {
  static struct utmp utmp_entry;

  if(getutent_r(utmp_path, &utmp_fp, &utmp_entry))
    return 0;
  return &utmp_entry;
}

void utmpname(const char *path) {
  if(strlen(path) >= sizeof utmp_path)
    abort();
  endutent();
}

#endif
